@@ -22,14 +22,20 @@
 
 #include <config.h>
 #include <stdio.h>
-#include <gnome.h>
-#include <bonobo.h>
+
+#include <libgnome/libgnome.h>
+#include <libbonobo.h>
+#include <libbonoboui.h>
+
 #include "gthumb-init.h"
 #include "image-viewer.h"
 #include "nav-window.h"
 #include "viewer-control.h"
-#include "stream.h"
+#include "viewer-nautilus-view.h"
+#include "viewer-stream.h"
 #include "ui.h"
+#include "gtk-utils.h"
+#include "iids.h"
 
 #include "nav_button.xpm"
 
@@ -55,7 +61,7 @@ enum {
 	ARG_ZOOM_CHANGE_KEEP_PREV,
 
 	ARG_TRANSP_TYPE_WHITE,
-	ARG_TRANSP_TYPE_GRAY,
+	ARG_TRANSP_TYPE_NONE,
 	ARG_TRANSP_TYPE_BLACK,
 	ARG_TRANSP_TYPE_CHECKED,
 
@@ -84,42 +90,43 @@ static BonoboUIVerb verbs [] = {
 	BONOBO_UI_VERB ("StartStopAnimation", verb_start_stop_ani),
 	BONOBO_UI_VERB ("StepAnimation",      verb_step_ani),
 
+	BONOBO_UI_VERB ("SaveImage",          verb_save_image),
+
 	BONOBO_UI_VERB_END
 };
 
 
 static void
-activate_control (ViewerControl *control)
+activate_control (ViewerControl *viewer_control)
 {
-	Bonobo_UIContainer remote_uicontainer;
-	BonoboUIComponent *ui_component;
-	BonoboControl *bcontrol = BONOBO_CONTROL (control);
+	Bonobo_UIContainer  ui_container;
+	BonoboUIComponent  *ui_component;
+	BonoboControl      *control = BONOBO_CONTROL (viewer_control);
 
-	if (bonobo_control_get_control_frame (bcontrol) == CORBA_OBJECT_NIL)
+	ui_container = bonobo_control_get_remote_ui_container (control, NULL);
+	if (ui_container == CORBA_OBJECT_NIL)
                 return;
 
-        remote_uicontainer = bonobo_control_get_remote_ui_container (bcontrol);
-	if (remote_uicontainer == CORBA_OBJECT_NIL)
-                return;
-
-        ui_component = bonobo_control_get_ui_component (bcontrol);
+        ui_component = bonobo_control_get_ui_component (control);
 	if (ui_component == CORBA_OBJECT_NIL)
                 return;
 
-        bonobo_ui_component_set_container (ui_component, remote_uicontainer);
-	bonobo_object_release_unref (remote_uicontainer, NULL);
+        bonobo_ui_component_set_container (ui_component, ui_container, NULL);
+	bonobo_object_release_unref (ui_container, NULL);
+
+	bonobo_ui_util_set_ui (ui_component, 
+			       GNOMEDATADIR,
+			       "GNOME_GThumb_Viewer.xml",
+			       "gthumb-image-viewer",
+			       NULL);
 
 	bonobo_ui_component_add_verb_list_with_data (ui_component, 
 						     verbs, 
 						     control);
-	bonobo_ui_util_set_ui (ui_component, 
-			       GNOMEDATADIR,
-			       "GNOME_GThumb_Viewer.xml",
-			       "gthumb-image-viewer");
 
-	setup_transparency_menu (control);
-	setup_zoom_quality_menu (control);
-	update_menu_sensitivity (control); 
+	setup_transparency_menu (viewer_control);
+	setup_zoom_quality_menu (viewer_control);
+	update_menu_sensitivity (viewer_control); 
 }
 
 
@@ -129,15 +136,17 @@ deactivate_control (BonoboControl *control)
 	BonoboUIComponent *ui_component;
 
 	ui_component = bonobo_control_get_ui_component (control);
+
 	if (ui_component == CORBA_OBJECT_NIL)
                 return;
 
-	bonobo_ui_component_unset_container (ui_component);
+	bonobo_ui_component_unset_container (ui_component, NULL);
 }
 
 
 static void
-viewer_control_activate (BonoboControl *object, gboolean state)
+viewer_control_activate (BonoboControl *object, 
+			 gboolean       state)
 {
 	BonoboControl *control;
 
@@ -155,12 +164,27 @@ viewer_control_activate (BonoboControl *object, gboolean state)
 }
 
 
+static void
+viewer_control_finalize (GObject *object)
+{
+	ViewerControl *control = VIEWER_CONTROL (object);
+
+	g_free (control->priv);
+	G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
+
 static void
 viewer_control_class_init (ViewerControl *klass)
 {
-	BonoboControlClass *control_class = (BonoboControlClass *)klass;
+	GObjectClass       *object_class;
+	BonoboControlClass *control_class;
 
-	parent_class = gtk_type_class (bonobo_control_get_type ());
+	parent_class  = g_type_class_peek_parent (klass);
+	object_class  = (GObjectClass *) klass;
+	control_class = (BonoboControlClass*) klass;
+
+	object_class->finalize = viewer_control_finalize;
 	control_class->activate = viewer_control_activate;
 }
 
@@ -168,49 +192,29 @@ viewer_control_class_init (ViewerControl *klass)
 static void
 viewer_control_init (ViewerControl *control)
 {
-	control->zoomable = NULL;
-	control->viewer   = NULL;
+	control->priv = g_new (ViewerControlPrivate, 1);
+	control->priv->zoomable = NULL;
+	control->priv->viewer   = NULL;
 }
 
 
-GtkType
-viewer_control_get_type (void)
-{
-	static GtkType type = 0;
-
-	if (!type) {
-		GtkTypeInfo info = {
-			"ViewerControl",
-			sizeof (ViewerControl),
-			sizeof (ViewerControlClass),
-			(GtkClassInitFunc)  viewer_control_class_init,
-			(GtkObjectInitFunc) viewer_control_init,
-			NULL, /* reserved 1 */
-			NULL, /* reserved 2 */
-			(GtkClassInitFunc) NULL
-		};
-
-		type = gtk_type_unique (bonobo_control_get_type (), &info);
-	}
-
-	return type;
-}
+BONOBO_TYPE_FUNC (ViewerControl, BONOBO_TYPE_CONTROL, viewer_control);
 
 
-static TranspType
+static GthTranspType
 get_transp_type_from_arg (guint arg_id)
 {
-	TranspType transp_type = TRANSP_TYPE_CHECKED;
+	GthTranspType transp_type = GTH_TRANSP_TYPE_CHECKED;
 
 	switch (arg_id) {
 	case ARG_TRANSP_TYPE_WHITE:
-		transp_type = TRANSP_TYPE_WHITE; break;
-	case ARG_TRANSP_TYPE_GRAY:
-		transp_type = TRANSP_TYPE_GRAY; break;
+		transp_type = GTH_TRANSP_TYPE_WHITE; break;
+	case ARG_TRANSP_TYPE_NONE:
+		transp_type = GTH_TRANSP_TYPE_NONE; break;
 	case ARG_TRANSP_TYPE_BLACK:
-		transp_type = TRANSP_TYPE_BLACK; break;
+		transp_type = GTH_TRANSP_TYPE_BLACK; break;
 	case ARG_TRANSP_TYPE_CHECKED:
-		transp_type = TRANSP_TYPE_CHECKED; break;
+		transp_type = GTH_TRANSP_TYPE_CHECKED; break;
 	default: break;
 	}
 
@@ -218,18 +222,18 @@ get_transp_type_from_arg (guint arg_id)
 }
 
 
-static CheckType
+static GthCheckType
 get_check_type_from_arg (guint arg_id)
 {
-	CheckType check_type = CHECK_TYPE_MIDTONE;
+	GthCheckType check_type = GTH_CHECK_TYPE_MIDTONE;
 
 	switch (arg_id) {
 	case ARG_CHECK_TYPE_LIGHT:
-		check_type = CHECK_TYPE_LIGHT; break;
+		check_type = GTH_CHECK_TYPE_LIGHT; break;
 	case ARG_CHECK_TYPE_MIDTONE:
-		check_type = CHECK_TYPE_MIDTONE; break;
+		check_type = GTH_CHECK_TYPE_MIDTONE; break;
 	case ARG_CHECK_TYPE_DARK:
-		check_type = CHECK_TYPE_DARK; break;
+		check_type = GTH_CHECK_TYPE_DARK; break;
 	default: break;
 	}
 
@@ -237,18 +241,18 @@ get_check_type_from_arg (guint arg_id)
 }
 
 
-static CheckSize
+static GthCheckSize
 get_check_size_from_arg (guint arg_id)
 {
-	CheckSize check_size = CHECK_SIZE_MEDIUM;
+	GthCheckSize check_size = GTH_CHECK_SIZE_MEDIUM;
 
 	switch (arg_id) {
 	case ARG_CHECK_SIZE_SMALL:
-		check_size = CHECK_SIZE_SMALL; break;
+		check_size = GTH_CHECK_SIZE_SMALL; break;
 	case ARG_CHECK_SIZE_MEDIUM:
-		check_size = CHECK_SIZE_MEDIUM; break;
+		check_size = GTH_CHECK_SIZE_MEDIUM; break;
 	case ARG_CHECK_SIZE_LARGE:
-		check_size = CHECK_SIZE_LARGE; break;
+		check_size = GTH_CHECK_SIZE_LARGE; break;
 	default: break;
 	}
 
@@ -258,19 +262,19 @@ get_check_size_from_arg (guint arg_id)
 
 static void
 get_prop (BonoboPropertyBag *bag, 
-	  BonoboArg *arg, 
-	  guint arg_id, 
+	  BonoboArg         *arg, 
+	  guint              arg_id, 
           CORBA_Environment *ev, 
-	  gpointer data)
+	  gpointer           data)
 {
 	ViewerControl *control = VIEWER_CONTROL (data);
-	ImageViewer *viewer = control->viewer;
-	TranspType transp_type;
-	CheckType check_type;
-	CheckSize check_size;
-	gint int_val;
-	gboolean bool_val;
-	gdouble double_val;
+	ImageViewer   *viewer = control->priv->viewer;
+	GthTranspType  transp_type;
+	GthCheckType   check_type;
+	GthCheckSize   check_size;
+	int            int_val;
+	gboolean       bool_val;
+	double         double_val;
 
         switch (arg_id) {
         case ARG_IS_VOID:
@@ -279,7 +283,7 @@ get_prop (BonoboPropertyBag *bag,
 		break;
 
 	case ARG_RUNNING_ANIMATION:
-		bool_val = image_viewer_is_running_animation (viewer);
+		bool_val = image_viewer_is_playing_animation (viewer);
 		BONOBO_ARG_SET_BOOLEAN (arg, bool_val);
 		break;
 
@@ -311,19 +315,19 @@ get_prop (BonoboPropertyBag *bag,
 		break;
 	case ARG_ZOOM_QUALITY_HIGH:
 		int_val = image_viewer_get_zoom_quality (viewer);
-		BONOBO_ARG_SET_BOOLEAN (arg, int_val == ZOOM_QUALITY_HIGH);
+		BONOBO_ARG_SET_BOOLEAN (arg, int_val == GTH_ZOOM_QUALITY_HIGH);
 		break;
 	case ARG_ZOOM_CHANGE_ACTUAL_SIZE:
 		int_val = image_viewer_get_zoom_change (viewer);
-		BONOBO_ARG_SET_BOOLEAN (arg, int_val == ZOOM_CHANGE_ACTUAL_SIZE);
+		BONOBO_ARG_SET_BOOLEAN (arg, int_val == GTH_ZOOM_CHANGE_ACTUAL_SIZE);
 		break;
 	case ARG_ZOOM_CHANGE_FIT:
 		int_val = image_viewer_get_zoom_change (viewer);
-		BONOBO_ARG_SET_BOOLEAN (arg, int_val == ZOOM_CHANGE_FIT);
+		BONOBO_ARG_SET_BOOLEAN (arg, int_val == GTH_ZOOM_CHANGE_FIT);
 		break;
 	case ARG_ZOOM_CHANGE_KEEP_PREV:
 		int_val = image_viewer_get_zoom_change (viewer);
-		BONOBO_ARG_SET_BOOLEAN (arg, int_val == ZOOM_CHANGE_KEEP_PREV);
+		BONOBO_ARG_SET_BOOLEAN (arg, int_val == GTH_ZOOM_CHANGE_KEEP_PREV);
 		break;
 	case ARG_ZOOM_FIT:
 		bool_val = image_viewer_is_zoom_to_fit (viewer);
@@ -331,7 +335,7 @@ get_prop (BonoboPropertyBag *bag,
 		break;
 
 	case ARG_TRANSP_TYPE_WHITE:
-	case ARG_TRANSP_TYPE_GRAY:
+	case ARG_TRANSP_TYPE_NONE:
 	case ARG_TRANSP_TYPE_BLACK:
 	case ARG_TRANSP_TYPE_CHECKED:
 		int_val = image_viewer_get_transp_type (viewer);
@@ -372,18 +376,18 @@ get_prop (BonoboPropertyBag *bag,
 
 static void
 set_prop (BonoboPropertyBag *bag, 
-	  const BonoboArg *arg, 
-	  guint arg_id, 
+	  const BonoboArg   *arg, 
+	  guint              arg_id, 
           CORBA_Environment *ev, 
-	  gpointer data)
+	  gpointer           data)
 {
 	ViewerControl *control = data;
-	ImageViewer *viewer = control->viewer;
-	gboolean bool_val;
-	gdouble double_val;
-	TranspType transp_type;
-	CheckType check_type;
-	CheckSize check_size;
+	ImageViewer   *viewer = control->priv->viewer;
+	gboolean       bool_val;
+	double         double_val;
+	GthTranspType  transp_type;
+	GthCheckType   check_type;
+	GthCheckSize   check_size;
 
         switch (arg_id) {
         case ARG_IS_VOID:
@@ -404,21 +408,21 @@ set_prop (BonoboPropertyBag *bag,
 		break;
 	case ARG_ZOOM_QUALITY_HIGH:
 		bool_val = BONOBO_ARG_GET_BOOLEAN (arg);
-		image_viewer_set_zoom_quality (viewer, bool_val ? ZOOM_QUALITY_HIGH : ZOOM_QUALITY_LOW);
-		image_viewer_update_view (viewer); /* FIXME */
+		image_viewer_set_zoom_quality (viewer, bool_val ? GTH_ZOOM_QUALITY_HIGH : GTH_ZOOM_QUALITY_LOW);
+		image_viewer_update_view (viewer);
 		break;
 	case ARG_ZOOM_CHANGE_ACTUAL_SIZE:
-		image_viewer_set_zoom_change (viewer, ZOOM_CHANGE_ACTUAL_SIZE);
+		image_viewer_set_zoom_change (viewer, GTH_ZOOM_CHANGE_ACTUAL_SIZE);
 		break;
 	case ARG_ZOOM_CHANGE_FIT:
-		image_viewer_set_zoom_change (viewer, ZOOM_CHANGE_FIT);
+		image_viewer_set_zoom_change (viewer, GTH_ZOOM_CHANGE_FIT);
 		break;
 	case ARG_ZOOM_CHANGE_KEEP_PREV:
-		image_viewer_set_zoom_change (viewer, ZOOM_CHANGE_KEEP_PREV);
+		image_viewer_set_zoom_change (viewer, GTH_ZOOM_CHANGE_KEEP_PREV);
 		break;
 	case ARG_ZOOM_FIT:
 		bool_val = BONOBO_ARG_GET_BOOLEAN (arg);
-		if (bool_val)
+		if (bool_val) 
 			image_viewer_zoom_to_fit (viewer);
 		else
 			image_viewer_set_zoom (viewer, 
@@ -426,12 +430,12 @@ set_prop (BonoboPropertyBag *bag,
 		break;
 
 	case ARG_TRANSP_TYPE_WHITE:
-	case ARG_TRANSP_TYPE_GRAY:
+	case ARG_TRANSP_TYPE_NONE:
 	case ARG_TRANSP_TYPE_BLACK:
 	case ARG_TRANSP_TYPE_CHECKED:
 		transp_type = get_transp_type_from_arg (arg_id);
 		image_viewer_set_transp_type (viewer, transp_type);
-		image_viewer_update_view (viewer); /* FIXME */
+		image_viewer_update_view (viewer);
 		break;
 	case ARG_CHECK_TYPE_LIGHT:
 	case ARG_CHECK_TYPE_MIDTONE:
@@ -439,14 +443,14 @@ set_prop (BonoboPropertyBag *bag,
 		check_type = get_check_type_from_arg (arg_id);
 		image_viewer_set_check_type (viewer, check_type);
 		image_viewer_set_transp_type (viewer, image_viewer_get_transp_type (viewer));
-		image_viewer_update_view (viewer); /* FIXME */
+		image_viewer_update_view (viewer);
 		break;
 	case ARG_CHECK_SIZE_SMALL:
 	case ARG_CHECK_SIZE_MEDIUM:
 	case ARG_CHECK_SIZE_LARGE:
 		check_size = get_check_size_from_arg (arg_id);
 		image_viewer_set_check_size (viewer, check_size);
-		image_viewer_update_view (viewer); /* FIXME */
+		image_viewer_update_view (viewer);
 		break;
 
 	case ARG_CURSOR_VISIBLE:
@@ -482,92 +486,142 @@ static const gchar *zoom_level_names[] = {
 static const gint n_zoom_levels = sizeof (zoom_levels) / sizeof (float);
 
 
-static int
-zoom_changed_cb (GtkWidget *widget,
+static void
+zoom_changed_cb (GtkWidget     *widget,
 		 ViewerControl *control)
 {
 	float new_zoom;	
 
-	if (! control->zoomable)
-		return FALSE;
+	if (! control->priv->zoomable)
+		return;
 
-	new_zoom = image_viewer_get_zoom (control->viewer);
-        bonobo_zoomable_report_zoom_level_changed (control->zoomable, 
-						   new_zoom);
+	new_zoom = image_viewer_get_zoom (control->priv->viewer);
+        bonobo_zoomable_report_zoom_level_changed (control->priv->zoomable, 
+						   new_zoom,
+						   NULL);
+}
 
-	return TRUE;
+
+static void
+size_changed_cb (GtkWidget     *widget, 
+		 ViewerControl *control)
+{
+	GtkAdjustment *vadj, *hadj;
+	gboolean       hide_vscr, hide_hscr;
+
+	vadj = IMAGE_VIEWER (control->priv->viewer)->vadj;
+	hadj = IMAGE_VIEWER (control->priv->viewer)->hadj;
+
+	hide_vscr = vadj->upper <= vadj->page_size;
+	hide_hscr = hadj->upper <= hadj->page_size;
+
+	if (hide_vscr && hide_hscr) {
+		if (! GTK_WIDGET_VISIBLE (control->priv->viewer_nav_btn))
+			return;
+
+		gtk_widget_hide (control->priv->viewer_vscr); 
+		gtk_widget_hide (control->priv->viewer_hscr); 
+		gtk_widget_hide (control->priv->viewer_nav_btn);
+		gtk_container_check_resize (GTK_CONTAINER (control->priv->viewer_table));
+	} else {
+		if (GTK_WIDGET_VISIBLE (control->priv->viewer_nav_btn))
+			return;
+
+		gtk_widget_show (control->priv->viewer_nav_btn);
+		gtk_widget_show (control->priv->viewer_vscr); 
+		gtk_widget_show (control->priv->viewer_hscr); 
+		gtk_container_check_resize (GTK_CONTAINER (control->priv->viewer_table));
+	}
 }
 
 
 static void
 zoomable_zoom_in_cb (BonoboZoomable *zoomable, 
-		     ViewerControl *control)
+		     ViewerControl  *control)
 {
-	image_viewer_zoom_in (control->viewer);
+	image_viewer_zoom_in (control->priv->viewer);
 }
 
 
 static void
 zoomable_zoom_out_cb (BonoboZoomable *zoomable, 
-		      ViewerControl *control)
+		      ViewerControl  *control)
 {
-	image_viewer_zoom_out (control->viewer);
+	image_viewer_zoom_out (control->priv->viewer);
 }
 
 
 static void
 zoomable_zoom_to_fit_cb (BonoboZoomable *zoomable, 
-			 ViewerControl *control)
+			 ViewerControl  *control)
 {
-	image_viewer_zoom_to_fit (control->viewer);
+	image_viewer_zoom_to_fit (control->priv->viewer);
 }
 
 
 static void
 zoomable_zoom_to_default_cb (BonoboZoomable *zoomable, 
-			     ViewerControl *control)
+			     ViewerControl  *control)
 {
-	image_viewer_set_zoom (control->viewer, 1.0);
+	image_viewer_set_zoom (control->priv->viewer, 1.0);
 }
 
 
 static void
 zoomable_set_zoom_level_cb (BonoboZoomable *zoomable, 
-			    float new_zoom_level,
-			    ViewerControl *control)
+			    float           new_zoom_level,
+			    ViewerControl  *control)
 {
-	image_viewer_set_zoom (control->viewer, new_zoom_level);
+	image_viewer_set_zoom (control->priv->viewer, new_zoom_level);
 }
 
 
 static GtkWidget *
 create_widgets (ViewerControl *control,
-		ImageViewer *viewer)
+		ImageViewer   *viewer)
 {
 	GtkWidget *hbox;
 	GtkWidget *table;
-	GtkWidget *hscrollbar;
-	GtkWidget *vscrollbar;
+	GtkWidget *hscr;
+	GtkWidget *vscr;
 	GtkWidget *event_box;
 	GtkWidget *pixmap;
 
+	/* set viewer signals. */
+
+	g_signal_connect (G_OBJECT (viewer), 
+			  "zoom_changed",
+			  G_CALLBACK (zoom_changed_cb), 
+			  control);
+
+	/* set viewer preferences. */
+
+	image_viewer_set_zoom_quality (viewer, pref_get_zoom_quality ());
+	image_viewer_set_zoom_change (viewer, pref_get_zoom_change ());
+	image_viewer_set_check_type (viewer, pref_get_check_type ());
+	image_viewer_set_check_size (viewer, pref_get_check_size ());
+	image_viewer_set_transp_type (viewer, pref_get_transp_type ());
+
+	/**/
+
 	hbox = gtk_hbox_new (FALSE, 0);
 	gtk_container_add (GTK_CONTAINER (hbox), GTK_WIDGET (viewer));
 
-	table = gtk_table_new (2, 2, FALSE);
-	vscrollbar = gtk_vscrollbar_new (viewer->vadj);
-	hscrollbar = gtk_hscrollbar_new (viewer->hadj);
-	event_box = gtk_event_box_new ();
-	pixmap = gnome_pixmap_new_from_xpm_d (nav_button_xpm);
+	control->priv->viewer_table = table = gtk_table_new (2, 2, FALSE);
+	control->priv->viewer_vscr = vscr = gtk_vscrollbar_new (viewer->vadj);
+	control->priv->viewer_hscr = hscr = gtk_hscrollbar_new (viewer->hadj);
+
+	control->priv->viewer_nav_btn = event_box = gtk_event_box_new ();
+	pixmap = _gtk_image_new_from_xpm_data (nav_button_xpm);
 	gtk_container_add (GTK_CONTAINER (event_box), pixmap);
 
 	gtk_table_attach (GTK_TABLE (table), hbox, 0, 1, 0, 1,
-			  (GtkAttachOptions) (GTK_FILL),
-			  (GtkAttachOptions) (GTK_FILL), 0, 0);
-	gtk_table_attach (GTK_TABLE (table), vscrollbar, 1, 2, 0, 1,
+			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
+	gtk_table_attach (GTK_TABLE (table), vscr, 1, 2, 0, 1,
 			  (GtkAttachOptions) (GTK_FILL),
 			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
-	gtk_table_attach (GTK_TABLE (table), hscrollbar, 0, 1, 1, 2,
+	gtk_table_attach (GTK_TABLE (table), hscr, 0, 1, 1, 2,
 			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
 			  (GtkAttachOptions) (GTK_FILL), 0, 0);
 	gtk_table_attach (GTK_TABLE (table), event_box, 1, 2, 1, 2,
@@ -576,22 +630,14 @@ create_widgets (ViewerControl *control,
 
         /* signals */
 
-	gtk_signal_connect (GTK_OBJECT (event_box), 
-			    "button_press_event",
-			    (GtkSignalFunc) nav_button_clicked_cb, 
-			    viewer);
-	gtk_signal_connect (GTK_OBJECT (viewer), 
-			    "zoom_changed",
-			    (GtkSignalFunc) zoom_changed_cb, 
-			    control);
-
-	/* set viewer preferences. */
-
-	image_viewer_set_zoom_quality (viewer, preferences.zoom_quality);
-	image_viewer_set_zoom_change (viewer, preferences.zoom_change);
-	image_viewer_set_check_type (viewer, preferences.check_type);
-	image_viewer_set_check_size (viewer, preferences.check_size);
-	image_viewer_set_transp_type (viewer, preferences.transp_type);
+	g_signal_connect (G_OBJECT (event_box), 
+			  "button_press_event",
+			  G_CALLBACK (nav_button_clicked_cb), 
+			  viewer);
+	g_signal_connect (G_OBJECT (viewer), 
+			  "size_changed",
+			  G_CALLBACK (size_changed_cb), 
+			  control);
 
 	gtk_widget_show_all (table);
 
@@ -601,37 +647,38 @@ create_widgets (ViewerControl *control,
 
 static void
 construct_control (ViewerControl *control,
-		   ImageViewer *viewer)
+		   ImageViewer   *viewer)
 {
-	BonoboPersistStream *stream;
-	BonoboPropertyBag *pbag;
-	BonoboZoomable *zoomable;
-	GtkWidget *root_widget;
+	BonoboObject       *stream;
+	BonoboPropertyBag  *pbag;
+	BonoboZoomable     *zoomable;
+	GtkWidget          *root_widget;
 	
-	g_return_if_fail (viewer != NULL);
 	g_return_if_fail (control != NULL);
+	g_return_if_fail (viewer != NULL);
 
 	/* create the control. */
 
-	control->viewer = viewer;
+	control->priv->viewer = viewer;
 	root_widget = create_widgets (control, viewer);
 	bonobo_control_construct (BONOBO_CONTROL (control), root_widget);
 
 	/* add the Bonobo::PersistStream interface. */
 
-	stream = bonobo_persist_stream_new (load_stream,
-					    NULL, /* no save */
-					    NULL, /* no getMaxSize */
-					    get_supported_types,
-					    control);
-
-	bonobo_object_add_interface (BONOBO_OBJECT (control),
-				     BONOBO_OBJECT (stream));
+	stream = viewer_stream_new (control);
+	bonobo_object_add_interface (BONOBO_OBJECT (control), stream);
 
 	/* add the Bonobo::PropertyBag interface. */
 
 	pbag = bonobo_property_bag_new (get_prop, set_prop, control);
-        bonobo_control_set_properties (BONOBO_CONTROL (control), pbag);
+        bonobo_control_set_properties (BONOBO_CONTROL (control), 
+				       BONOBO_OBJREF (pbag), 
+				       NULL);
+
+	/* add the Nautilus::View interface. */
+
+	control->priv->nautilus_view = viewer_nautilus_view_new (control);
+	bonobo_object_add_interface (BONOBO_OBJECT (control), control->priv->nautilus_view);
 
 	/* - viewer content */
 
@@ -763,11 +810,11 @@ construct_control (ViewerControl *control,
 				  | BONOBO_PROPERTY_WRITEABLE));
 
         bonobo_property_bag_add (pbag, 
-				 "transp_type_gray",
-				 ARG_TRANSP_TYPE_GRAY,
+				 "transp_type_none",
+				 ARG_TRANSP_TYPE_NONE,
 				 BONOBO_ARG_BOOLEAN, 
 				 NULL, 
-				 "Use gray color as transparency "
+				 "Use gtk theme background as transparency "
 				 "indicator",
 				 (BONOBO_PROPERTY_READABLE 
 				  | BONOBO_PROPERTY_WRITEABLE));
@@ -873,23 +920,28 @@ construct_control (ViewerControl *control,
 	/* add the Bonobo::Zoomable interface. */
 	
 	zoomable = bonobo_zoomable_new ();
-	control->zoomable = zoomable;
+	control->priv->zoomable = zoomable;
 
-        gtk_signal_connect (GTK_OBJECT (zoomable), "set_zoom_level",
-                            GTK_SIGNAL_FUNC (zoomable_set_zoom_level_cb),
-			    control);
-        gtk_signal_connect (GTK_OBJECT (zoomable), "zoom_in",
-                            GTK_SIGNAL_FUNC (zoomable_zoom_in_cb), 
-			    control);
-        gtk_signal_connect (GTK_OBJECT (zoomable), "zoom_out",
-                            GTK_SIGNAL_FUNC (zoomable_zoom_out_cb), 
-			    control);
-        gtk_signal_connect (GTK_OBJECT (zoomable), "zoom_to_fit",
-                            GTK_SIGNAL_FUNC (zoomable_zoom_to_fit_cb), 
-			    control);
-        gtk_signal_connect (GTK_OBJECT (zoomable), "zoom_to_default",
-                            GTK_SIGNAL_FUNC (zoomable_zoom_to_default_cb), 
-			    control);
+	g_signal_connect (G_OBJECT (zoomable), 
+			  "set_zoom_level",
+			  G_CALLBACK (zoomable_set_zoom_level_cb),
+			  control);
+	g_signal_connect (G_OBJECT (zoomable), 
+			  "zoom_in",
+			  G_CALLBACK (zoomable_zoom_in_cb), 
+			  control);
+	g_signal_connect (G_OBJECT (zoomable), 
+			  "zoom_out",
+			  G_CALLBACK (zoomable_zoom_out_cb), 
+			  control);
+	g_signal_connect (G_OBJECT (zoomable), 
+			  "zoom_to_fit",
+			  G_CALLBACK (zoomable_zoom_to_fit_cb), 
+			  control);
+	g_signal_connect (G_OBJECT (zoomable), 
+			  "zoom_to_default",
+			  G_CALLBACK (zoomable_zoom_to_default_cb), 
+			  control);
 
         bonobo_zoomable_set_parameters_full (zoomable,
                                              1.0,
@@ -901,7 +953,6 @@ construct_control (ViewerControl *control,
                                              zoom_levels,
                                              zoom_level_names,
                                              n_zoom_levels);
-
         bonobo_object_add_interface (BONOBO_OBJECT (control),
                                      BONOBO_OBJECT (zoomable));
 }
@@ -910,12 +961,12 @@ construct_control (ViewerControl *control,
 BonoboControl *
 viewer_control_new (ImageViewer *viewer)
 {
-	ViewerControl *control;
+	BonoboControl *control;
 	
 	g_return_val_if_fail (viewer != NULL, NULL);
 
-	control = gtk_type_new (viewer_control_get_type ());
-	construct_control (control, viewer);
+	control = g_object_new (VIEWER_TYPE_CONTROL, NULL);
+	construct_control (VIEWER_CONTROL (control), viewer);
 
-	return BONOBO_CONTROL (control);
+	return control;
 }
