@@ -3,7 +3,7 @@
 /*
  *  GThumb
  *
- *  Copyright (C) 2003 Free Software Foundation, Inc.
+ *  Copyright (C) 2003, 2004, 2005 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -32,7 +32,9 @@
 #include <gtk/gtk.h>
 #include <libgnomeui/gnome-icon-theme.h>
 #include <libgnomeui/gnome-icon-lookup.h>
+#include <libgnomevfs/gnome-vfs-file-info.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
+#include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <gphoto2/gphoto2-context.h>
 #include <gphoto2/gphoto2-camera.h>
@@ -51,9 +53,9 @@
 #include "gconf-utils.h"
 #include "preferences.h"
 #include "pixbuf-utils.h"
+#include "rotation-utils.h"
 #include "main.h"
 
-
 #define GLADE_FILE "gthumb_camera.glade"
 #define CATEGORIES_GLADE_FILE "gthumb_comment.glade"
 #define CAMERA_FILE "gphoto-48.png"
@@ -91,6 +93,7 @@ typedef struct {
 	GtkWidget      *import_preview_box;
 	GtkWidget      *import_reload_button;
 	GtkWidget      *import_delete_button;
+	GtkWidget      *import_ok_button;
 	GtkWidget      *i_commands_table;
 
 	GtkWidget      *progress_info_image;
@@ -111,6 +114,7 @@ typedef struct {
 
 	gboolean             keep_original_filename;
 	gboolean             delete_from_camera;
+	gboolean             adjust_orientation;
 
 	int                  image_n;
 	char                *local_folder;
@@ -128,6 +132,7 @@ typedef struct {
 
 	GList               *categories_list;
 	GList               *delete_list;
+	GList               *adjust_orientation_list;
 	GList               *saved_images_list;
 
 	/**/
@@ -138,6 +143,7 @@ typedef struct {
 	gboolean             thread_done; 
 
 	guint                idle_id;
+	
 } DialogData;
 
 
@@ -146,8 +152,8 @@ static void
 destroy_cb (GtkWidget  *widget, 
 	    DialogData *data)
 {
-	GthBrowser *browser = data->browser;
-	gboolean    thread_done;
+	GtkWidget *browser = (GtkWidget*) data->browser;
+	gboolean   thread_done;
 
 	/* Remove check. */
 
@@ -174,8 +180,13 @@ destroy_cb (GtkWidget  *widget,
 
 	/**/
 
-	if (data->view_folder)
-		gth_browser_go_to_directory (data->browser, data->local_folder);
+	if (data->view_folder) {
+		if (browser == NULL) {
+			browser = gth_browser_new (data->local_folder);
+			gtk_widget_show (browser);
+		} else
+			gth_browser_go_to_directory (data->browser, data->local_folder);
+	}
 
 	/**/
 
@@ -189,6 +200,7 @@ destroy_cb (GtkWidget  *widget,
 		g_object_unref (data->camera_present_pixbuf);
 	path_list_free (data->categories_list);
 	path_list_free (data->delete_list);
+	path_list_free (data->adjust_orientation_list);
 	path_list_free (data->saved_images_list);
 	gp_camera_unref (data->camera);
 	gp_context_unref (data->context);
@@ -201,7 +213,10 @@ destroy_cb (GtkWidget  *widget,
 
 	if (ImportPhotos) {
 		ImportPhotos = FALSE;
-		gth_window_close (GTH_WINDOW (browser));
+		if (browser != NULL)
+			gth_window_close (GTH_WINDOW (browser));
+		else
+			gtk_main_quit ();
 	}
 }
 
@@ -228,6 +243,7 @@ ctx_progress_start_func (GPContext  *context,
 			 gpointer    callback_data)
 {
 	DialogData *data = callback_data;
+	char *locale_string;
 
 	g_mutex_lock (data->yes_or_no);
 	data->update_ui = TRUE;
@@ -236,7 +252,9 @@ ctx_progress_start_func (GPContext  *context,
 	data->fraction = 0.0;
 	if (data->progress_info != NULL)
 		g_free (data->progress_info);
-	data->progress_info = g_strdup_vprintf (format, args);
+	locale_string = g_strdup_vprintf (format, args);
+	data->progress_info = g_locale_to_utf8 (locale_string, -1, NULL, NULL, NULL);
+	g_free (locale_string);
 	g_mutex_unlock (data->yes_or_no);
 
 	return data->current_op;
@@ -498,6 +516,7 @@ main_dialog_set_sensitive (DialogData *data,
 			   gboolean    value)
 {
 	gtk_widget_set_sensitive (data->select_model_button, value);
+	gtk_widget_set_sensitive (data->import_ok_button, value);
 	gtk_widget_set_sensitive (data->import_reload_button, value);
 	gtk_widget_set_sensitive (data->import_delete_button, value);
 	gtk_widget_set_sensitive (data->i_commands_table, value);
@@ -959,13 +978,16 @@ ctx_error_func (GPContext  *context,
 		gpointer    callback_data)
 {
 	DialogData *data = callback_data;
+	char *locale_string;
 
 	g_mutex_lock (data->yes_or_no);
 	data->update_ui = TRUE;
 	data->error = TRUE;
 	if (data->msg_text != NULL)
 		g_free (data->msg_text);
-	data->msg_text = g_strdup_vprintf (format, args);
+	locale_string = g_strdup_vprintf (format, args);
+	data->msg_text = g_locale_to_utf8 (locale_string, -1, NULL, NULL, NULL);
+	g_free (locale_string);
 	data->msg_icon = GTK_STOCK_DIALOG_ERROR;
 	g_mutex_unlock (data->yes_or_no);
 }
@@ -978,12 +1000,15 @@ ctx_status_func (GPContext  *context,
 		 gpointer    callback_data)
 {
 	DialogData *data = callback_data;
+	char *locale_string;
 
 	g_mutex_lock (data->yes_or_no);
 	data->update_ui = TRUE;
 	if (data->msg_text != NULL)
 		g_free (data->msg_text);
-	data->msg_text = g_strdup_vprintf (format, args);
+	locale_string = g_strdup_vprintf (format, args);	
+	data->msg_text = g_locale_to_utf8 (locale_string, -1, NULL, NULL, NULL);
+	g_free (locale_string);
 	data->msg_icon = GTK_STOCK_DIALOG_INFO;
 	g_mutex_unlock (data->yes_or_no);
 }
@@ -996,12 +1021,15 @@ ctx_message_func (GPContext  *context,
 		  gpointer    callback_data)
 {
 	DialogData *data = callback_data;
+	char *locale_string;
 
 	g_mutex_lock (data->yes_or_no);
 	data->update_ui = TRUE;
 	if (data->msg_text != NULL)
 		g_free (data->msg_text);
-	data->msg_text = g_strdup_vprintf (format, args);
+	locale_string = g_strdup_vprintf (format, args);
+	data->msg_text = g_locale_to_utf8 (locale_string, -1, NULL, NULL, NULL);
+	g_free (locale_string);
 	data->msg_icon = GTK_STOCK_DIALOG_WARNING;
 	g_mutex_unlock (data->yes_or_no);
 }
@@ -1043,7 +1071,7 @@ get_folder_name (DialogData *data)
 	char *film_name;
 	char *path;
 
-	esc_destination = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (data->destination_filechooserbutton));
+	esc_destination = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (data->destination_filechooserbutton));
 	destination = gnome_vfs_unescape_string (esc_destination, "");
 	g_free (esc_destination);
 
@@ -1198,6 +1226,9 @@ save_image (DialogData *data,
 	if (gp_file_save (file, local_path) >= 0) {
 		if (data->delete_from_camera) 
 			data->delete_list = g_list_prepend (data->delete_list, g_strdup (camera_path));
+		if (data->adjust_orientation) {
+			data->adjust_orientation_list = g_list_prepend (data->adjust_orientation_list, g_strdup (local_path));
+		}
 		data->saved_images_list = g_list_prepend (data->saved_images_list, g_strdup (camera_path));
 		add_categories_to_image (data, local_path);
 	} else {
@@ -1264,39 +1295,13 @@ delete_images__done (AsyncOperationData *aodata,
 
 	data->view_folder = TRUE;
 
-	if (ImportPhotos) {
-		ImportPhotos = FALSE;
-		gtk_widget_show (GTK_WIDGET (data->browser));
-	}
+        if (ImportPhotos) {
+                ImportPhotos = FALSE;
+		if (data->browser != NULL)
+			gtk_widget_show (GTK_WIDGET (data->browser));
+        }
 
-	gtk_widget_destroy (data->dialog);
-}
-
-
-static void 
-save_images__init (AsyncOperationData *aodata, 
-		   DialogData         *data)
-{
-	all_windows_remove_monitor ();
-
-	data->image_n = 1;
-	if (data->delete_list != NULL) {
-		path_list_free (data->delete_list);
-		data->delete_list = NULL;
-	}
-	if (data->saved_images_list != NULL) {
-		path_list_free (data->saved_images_list);
-		data->saved_images_list = NULL;
-	}
-}
-
-
-static void 
-save_images__step (AsyncOperationData *aodata, 
-		   DialogData         *data)
-{
-	const char *camera_path = aodata->scan->data;
-	save_image (data, camera_path, data->local_folder, data->image_n++);
+        gtk_widget_destroy (data->dialog);
 }
 
 
@@ -1320,6 +1325,90 @@ notify_cb (gpointer cb_data)
 }
 
 
+static void 
+adjust_orientation__done (AsyncOperationData *aodata,
+			  DialogData         *data)
+{
+	gboolean            interrupted;
+	AsyncOperationData *new_aodata;
+
+	g_mutex_lock (data->yes_or_no);
+	interrupted = data->interrupted;
+	g_mutex_unlock (data->yes_or_no);
+
+	data->idle_id = g_idle_add (notify_cb, data);
+
+	if (interrupted)
+		return;
+
+	new_aodata = async_operation_new (data->delete_list,
+					  NULL,
+					  delete_images__step,
+					  delete_images__done,
+					  data);
+	async_operation_start (new_aodata);
+}
+
+
+static void 
+adjust_orientation__step (AsyncOperationData *aodata,
+			  DialogData         *data)
+{
+	const char       *filepath = aodata->scan->data;
+	GnomeVFSFileInfo  info;
+	GtkWindow        *window = GTK_WINDOW (data->dialog);
+	
+	gnome_vfs_get_file_info (filepath, &info, GNOME_VFS_FILE_INFO_GET_ACCESS_RIGHTS|GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
+	
+	if (file_is_image (filepath, TRUE)) {
+		FileData     *fd = file_data_new (filepath, &info);
+		RotationData *rot_data = rotation_data_new ();
+
+		update_rotation_from_exif_data (fd->path, rot_data);
+		if (image_is_jpeg (filepath))
+			apply_transformation_jpeg (window, fd->path, rot_data);
+		else
+			apply_transformation_generic (window, fd->path, rot_data);
+
+		file_data_unref (fd);
+		g_free (rot_data);
+	}
+	
+	gnome_vfs_set_file_info (filepath, &info, GNOME_VFS_SET_FILE_INFO_PERMISSIONS|GNOME_VFS_SET_FILE_INFO_OWNER);
+}
+
+
+static void 
+save_images__init (AsyncOperationData *aodata, 
+		   DialogData         *data)
+{
+	all_windows_remove_monitor ();
+
+	data->image_n = 1;
+	if (data->delete_list != NULL) {
+		path_list_free (data->delete_list);
+		data->delete_list = NULL;
+	}
+	if (data->adjust_orientation_list != NULL) {
+		path_list_free (data->adjust_orientation_list);
+		data->adjust_orientation_list = NULL;
+	}
+	if (data->saved_images_list != NULL) {
+		path_list_free (data->saved_images_list);
+		data->saved_images_list = NULL;
+	}
+}
+
+
+static void 
+save_images__step (AsyncOperationData *aodata, 
+		   DialogData         *data)
+{
+	const char *camera_path = aodata->scan->data;
+	save_image (data, camera_path, data->local_folder, data->image_n++);
+}
+
+
 static void 
 save_images__done (AsyncOperationData *aodata, 
 		   DialogData         *data)
@@ -1333,15 +1422,13 @@ save_images__done (AsyncOperationData *aodata,
 	error = data->error;
 	g_mutex_unlock (data->yes_or_no);
 
-	data->idle_id = g_idle_add (notify_cb, data);
-
 	if (interrupted || error)
 		return;
 
-	new_aodata = async_operation_new (data->delete_list,
+	new_aodata = async_operation_new (data->adjust_orientation_list,
 					  NULL,
-					  delete_images__step,
-					  delete_images__done,
+					  adjust_orientation__step,
+					  adjust_orientation__done,
 					  data);
 	async_operation_start (new_aodata);
 }
@@ -1393,6 +1480,7 @@ ok_clicked_cb (GtkButton  *button,
 
 	data->keep_original_filename = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->keep_names_checkbutton));
 	data->delete_from_camera = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->delete_checkbutton));
+	data->adjust_orientation = eel_gconf_get_boolean (PREF_PHOTO_IMPORT_ADJUST_ORIENTATION, TRUE);
 
 	eel_gconf_set_boolean (PREF_PHOTO_IMPORT_KEEP_FILENAMES, data->keep_original_filename);
 	eel_gconf_set_boolean (PREF_PHOTO_IMPORT_DELETE, data->delete_from_camera);
@@ -1417,7 +1505,7 @@ ok_clicked_cb (GtkButton  *button,
 	if (sel_list != NULL) {
 		for (scan = sel_list; scan; scan = scan->next) {
 			FileData   *fdata = scan->data;
-			const char *filename = fdata->path;
+			const char *filename = file_data_local_path (fdata);
 			file_list = g_list_prepend (file_list, g_strdup (filename));
 		}
 		if (file_list != NULL)
@@ -1562,7 +1650,7 @@ import_delete_cb (GtkButton  *button,
 	if (sel_list != NULL) {
 		for (scan = sel_list; scan; scan = scan->next) {
 			FileData   *fdata = scan->data;
-			const char *filename = fdata->path;
+			const char *filename = file_data_local_path (fdata);
 			delete_list = g_list_prepend (delete_list, g_strdup (filename));
 		}
 		if (delete_list != NULL)
@@ -1689,7 +1777,7 @@ void
 dlg_photo_importer (GthBrowser *browser)
 {
 	DialogData   *data;
-	GtkWidget    *btn_ok, *btn_cancel;
+	GtkWidget    *btn_cancel;
 	GdkPixbuf    *mute_pixbuf;
 	char         *default_path;
 	char         *esc_uri;
@@ -1751,7 +1839,7 @@ dlg_photo_importer (GthBrowser *browser)
 	data->import_reload_button = glade_xml_get_widget (data->gui, "import_reload_button");
 	data->import_delete_button = glade_xml_get_widget (data->gui, "import_delete_button");
 	data->i_commands_table = glade_xml_get_widget (data->gui, "i_commands_table");
-	btn_ok = glade_xml_get_widget (data->gui, "import_okbutton");
+	data->import_ok_button = glade_xml_get_widget (data->gui, "import_okbutton");
 	btn_cancel = glade_xml_get_widget (data->gui, "import_cancelbutton");
 
 	data->image_list = gth_image_list_new (THUMB_SIZE + THUMB_BORDER);
@@ -1807,7 +1895,7 @@ dlg_photo_importer (GthBrowser *browser)
 			  "destroy",
 			  G_CALLBACK (destroy_cb),
 			  data);
-	g_signal_connect (G_OBJECT (btn_ok), 
+	g_signal_connect (G_OBJECT (data->import_ok_button), 
 			  "clicked",
 			  G_CALLBACK (ok_clicked_cb),
 			  data);
@@ -1835,7 +1923,8 @@ dlg_photo_importer (GthBrowser *browser)
 
 	/* run dialog. */
 
-	gtk_window_set_transient_for (GTK_WINDOW (data->dialog), GTK_WINDOW (browser));
+	if (browser != NULL)
+		gtk_window_set_transient_for (GTK_WINDOW (data->dialog), GTK_WINDOW (browser));
 	gtk_window_set_modal (GTK_WINDOW (data->dialog), FALSE);
 	gtk_widget_show (data->dialog);
 
@@ -2063,3 +2152,4 @@ dlg_select_camera_model_cb (GtkButton  *button,
 
 
 #endif /*HAVE_LIBGPHOTO */
+
