@@ -3,7 +3,7 @@
 /*
  *  GThumb
  *
- *  Copyright (C) 2001, 2003 Free Software Foundation, Inc.
+ *  Copyright (C) 2001-2005 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,13 +27,12 @@
 #include <fcntl.h>
 #include <unistd.h>
 
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <libgnome/libgnome.h>
-#include <libgnomeui/gnome-dialog.h>
-#include <libgnomeui/gnome-dialog-util.h>
-#include <libgnomeui/gnome-propertybox.h>
-#include <libgnomeui/gnome-pixmap.h>
+#include <libgnome/gnome-help.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
 #include <glade/glade.h>
+
 #include "thumb-cache.h"
 #include "comments.h"
 #include "file-utils.h"
@@ -42,7 +41,7 @@
 #include "gconf-utils.h"
 #include "gtk-utils.h"
 #include "typedefs.h"
-#include "gthumb-window.h"
+#include "gth-browser.h"
 
 #include "icons/layout1.xpm"
 #include "icons/layout2.xpm"
@@ -53,47 +52,49 @@ static gint thumb_size[] = {48, 64, 85, 95, 112, 128, 164, 200, 256};
 static gint thumb_sizes = sizeof (thumb_size) / sizeof (gint);
 
 #define GLADE_PREF_FILE "gthumb_preferences.glade"
+#define VIEW_AS_DELAY 500
 
 typedef struct {
-	GThumbWindow    *window;
+	GthBrowser *browser;
 
-	GladeXML  *gui;
-	GtkWidget *dialog;
+	GladeXML   *gui;
+	GtkWidget  *dialog;
 
-	GtkWidget *radio_current_location;
-	GtkWidget *radio_last_location;
-	GtkWidget *radio_use_startup;
-	GtkWidget *startup_dir_entry;
-	GtkWidget *file_entry;
-	GtkWidget *btn_set_to_current;
-	GtkWidget *radio_layout1;
-	GtkWidget *radio_layout2;
-	GtkWidget *radio_layout3;
-	GtkWidget *radio_layout4;
-	GtkWidget *toolbar_style_optionmenu;
+	GtkWidget  *radio_current_location;
+	GtkWidget  *radio_last_location;
+	GtkWidget  *radio_use_startup;
+	GtkWidget  *startup_dir_filechooserbutton;
+	GtkWidget  *btn_set_to_current;
+	GtkWidget  *radio_layout1;
+	GtkWidget  *radio_layout2;
+	GtkWidget  *radio_layout3;
+	GtkWidget  *radio_layout4;
+	GtkWidget  *toolbar_style_optionmenu;
 
-	GtkWidget *view_as_slides_radiobutton;
-	GtkWidget *view_as_list_radiobutton;
-	/*GtkWidget *toggle_show_hidden;*/
-	GtkWidget *toggle_show_filenames;
-	GtkWidget *toggle_show_comments;
-	GtkWidget *toggle_show_thumbs;
-	GtkWidget *toggle_file_type;
-	GtkWidget *opt_thumbs_size;
-	GtkWidget *toggle_confirm_del;
-	GtkWidget *toggle_ask_to_save;
-	GtkWidget *opt_click_policy;
+	GtkWidget  *view_as_slides_radiobutton;
+	GtkWidget  *view_as_list_radiobutton;
+	GtkWidget  *toggle_show_filenames;
+	GtkWidget  *toggle_show_comments;
+	GtkWidget  *toggle_show_thumbs;
+	GtkWidget  *toggle_file_type;
+	GtkWidget  *opt_thumbs_size;
+	GtkWidget  *toggle_confirm_del;
+	GtkWidget  *toggle_ask_to_save;
+	GtkWidget  *opt_click_policy;
 
-	GtkWidget *opt_zoom_quality_high;
-	GtkWidget *opt_zoom_quality_low;
-	GtkWidget *opt_zoom_change;
-	GtkWidget *opt_transparency;
+	GtkWidget  *opt_zoom_quality_high;
+	GtkWidget  *opt_zoom_quality_low;
+	GtkWidget  *opt_zoom_change;
+	GtkWidget  *opt_transparency;
 
-	GtkWidget *radio_ss_direction_forward;
-	GtkWidget *radio_ss_direction_reverse;
-	GtkWidget *spin_ss_delay;
-	GtkWidget *toggle_ss_wrap_around;
-	GtkWidget *toggle_ss_fullscreen;
+	GtkWidget  *radio_ss_direction_forward;
+	GtkWidget  *radio_ss_direction_reverse;
+	GtkWidget  *radio_ss_direction_random;
+	GtkWidget  *spin_ss_delay;
+	GtkWidget  *toggle_ss_wrap_around;
+
+	GthViewAs   new_view_type;
+	guint       view_as_timeout;
 } DialogData;
 
 
@@ -102,6 +103,8 @@ static void
 destroy_cb (GtkWidget *widget, 
 	    DialogData *data)
 {
+	if (data->view_as_timeout != 0)
+		g_source_remove (data->view_as_timeout);
 	g_object_unref (G_OBJECT (data->gui));
 	g_free (data);
 }
@@ -112,25 +115,23 @@ static void
 apply_cb (GtkWidget  *widget, 
 	  DialogData *data)
 {
+	GthDirectionType direction = GTH_DIRECTION_FORWARD;
+
 	/* Startup dir. */
 
 	eel_gconf_set_boolean (PREF_GO_TO_LAST_LOCATION, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->radio_last_location)));
 	eel_gconf_set_boolean (PREF_USE_STARTUP_LOCATION, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->radio_use_startup)));
 
 	if (eel_gconf_get_boolean (PREF_USE_STARTUP_LOCATION, FALSE)) {
-		char *temp;
-		char *text;
+		char *esc_path;
 		char *location;
 
-		text = _gtk_entry_get_filename_text (GTK_ENTRY (data->startup_dir_entry));
-		temp = remove_ending_separator (text);
-		location = g_strconcat ("file://", temp, NULL);
+		esc_path = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (data->startup_dir_filechooserbutton));
+		location = gnome_vfs_unescape_string (esc_path, "");
+		g_free (esc_path);
 
 		eel_gconf_set_path (PREF_STARTUP_LOCATION, location);
 		preferences_set_startup_location (location);
-
-		g_free (temp);
-		g_free (text);
 		g_free (location);
 	}
 
@@ -138,11 +139,18 @@ apply_cb (GtkWidget  *widget,
 
 	eel_gconf_set_boolean (PREF_SLIDESHOW_WRAP_AROUND, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->toggle_ss_wrap_around)));
 
-	eel_gconf_set_boolean (PREF_SLIDESHOW_FULLSCREEN, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->toggle_ss_fullscreen)));
-
 	eel_gconf_set_integer (PREF_SLIDESHOW_DELAY, gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (data->spin_ss_delay)));
 
-	pref_set_slideshow_direction (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->radio_ss_direction_forward)) ? GTH_DIRECTION_FORWARD : GTH_DIRECTION_REVERSE);
+	/**/
+
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->radio_ss_direction_forward)))
+		direction = GTH_DIRECTION_FORWARD;
+	else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->radio_ss_direction_reverse)))
+		direction = GTH_DIRECTION_REVERSE;
+	else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->radio_ss_direction_random)))
+		direction = GTH_DIRECTION_RANDOM;
+
+	pref_set_slideshow_direction (direction);
 }
 
 
@@ -194,7 +202,7 @@ static void
 use_startup_toggled_cb (GtkWidget *widget, 
 			DialogData *data)
 {
-	gtk_widget_set_sensitive (data->file_entry,
+	gtk_widget_set_sensitive (data->startup_dir_filechooserbutton,
 				  GTK_TOGGLE_BUTTON (widget)->active);
 	gtk_widget_set_sensitive (data->btn_set_to_current,
 				  GTK_TOGGLE_BUTTON (widget)->active);
@@ -206,11 +214,11 @@ static void
 set_to_current_cb (GtkWidget  *widget, 
 		   DialogData *data)
 {
-	if (data->window->dir_list->path == NULL)
-		return;
+	char *esc_uri;
 
-	_gtk_entry_set_filename_text (GTK_ENTRY (data->startup_dir_entry), 
-				      data->window->dir_list->path);
+	esc_uri = gnome_vfs_escape_host_and_path_string (gth_browser_get_current_directory (data->browser));
+	gtk_file_chooser_set_uri (GTK_FILE_CHOOSER (data->startup_dir_filechooserbutton), esc_uri);
+	g_free (esc_uri);
 }
 
 
@@ -278,14 +286,17 @@ show_comments_toggled_cb (GtkToggleButton *button,
 }
 
 
-/* FIXME
-static void
-show_hidden_toggled_cb (GtkToggleButton *button, 
-			DialogData      *data)
+static gboolean
+view_as_cb (gpointer cb_data)
 {
-	eel_gconf_set_boolean (PREF_SHOW_HIDDEN_FILES, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->toggle_show_hidden)));
+	DialogData *data = cb_data;
+
+	g_source_remove (data->view_as_timeout);
+	pref_set_view_as (data->new_view_type);
+	data->view_as_timeout = 0;
+
+	return FALSE;
 }
-*/
 
 
 static void
@@ -294,7 +305,10 @@ view_as_slides_toggled_cb (GtkToggleButton *button,
 {
 	if (! gtk_toggle_button_get_active (button))
 		return;
-	pref_set_view_as (GTH_VIEW_AS_THUMBNAILS);
+	if (data->view_as_timeout != 0)
+		g_source_remove (data->view_as_timeout);
+	data->new_view_type = GTH_VIEW_AS_THUMBNAILS;
+	data->view_as_timeout = g_timeout_add (VIEW_AS_DELAY, view_as_cb, data);
 }
 
 
@@ -304,7 +318,10 @@ view_as_list_toggled_cb (GtkToggleButton *button,
 {
 	if (! gtk_toggle_button_get_active (button))
 		return;
-	pref_set_view_as (GTH_VIEW_AS_LIST);
+	if (data->view_as_timeout != 0)
+		g_source_remove (data->view_as_timeout);
+	data->new_view_type = GTH_VIEW_AS_LIST;
+	data->view_as_timeout = g_timeout_add (VIEW_AS_DELAY, view_as_cb, data);
 }
 
 
@@ -392,17 +409,17 @@ transp_type_changed_cb (GtkOptionMenu *option_menu,
 
 /* create the main dialog. */
 void
-dlg_preferences (GThumbWindow *window)
+dlg_preferences (GthBrowser *browser)
 {
-	DialogData *data;
-	GtkWidget  *btn_close;
-	GtkWidget  *btn_help;
-	/*	int         i = 0;*/
-	int         layout_type;
-	char       *startup_location;
+	DialogData       *data;
+	GtkWidget        *btn_close;
+	GtkWidget        *btn_help;
+	int               layout_type;
+	char             *startup_location;
+	GthDirectionType  direction;
 
-	data = g_new (DialogData, 1);
-	data->window = window;
+	data = g_new0 (DialogData, 1);
+	data->browser = browser;
 	data->gui = glade_xml_new (GTHUMB_GLADEDIR "/" GLADE_PREF_FILE, NULL, NULL);
         if (!data->gui) {
                 g_warning ("Could not find " GLADE_PREF_FILE "\n");
@@ -421,8 +438,7 @@ dlg_preferences (GThumbWindow *window)
         data->radio_current_location = glade_xml_get_widget (data->gui, "radio_current_location");
         data->radio_last_location = glade_xml_get_widget (data->gui, "radio_last_location");
         data->radio_use_startup = glade_xml_get_widget (data->gui, "radio_use_startup");
-        data->startup_dir_entry = glade_xml_get_widget (data->gui, "startup_dir_entry");
-        data->file_entry = glade_xml_get_widget (data->gui, "file_entry");
+        data->startup_dir_filechooserbutton = glade_xml_get_widget (data->gui, "startup_dir_filechooserbutton");
 	data->btn_set_to_current = glade_xml_get_widget (data->gui, "btn_set_to_current");
         data->toggle_confirm_del = glade_xml_get_widget (data->gui, "toggle_confirm_del");
         data->toggle_ask_to_save = glade_xml_get_widget (data->gui, "toggle_ask_to_save");
@@ -435,10 +451,6 @@ dlg_preferences (GThumbWindow *window)
 	data->view_as_slides_radiobutton = glade_xml_get_widget (data->gui, "view_as_slides_radiobutton");
 	data->view_as_list_radiobutton = glade_xml_get_widget (data->gui, "view_as_list_radiobutton");
 
-	/* FIXME
-        data->toggle_show_hidden = glade_xml_get_widget (data->gui, "toggle_show_hidden");
-	*/
-
         data->toggle_show_filenames = glade_xml_get_widget (data->gui, "toggle_show_filenames");
         data->toggle_show_comments = glade_xml_get_widget (data->gui, "toggle_show_comments");
 
@@ -455,15 +467,15 @@ dlg_preferences (GThumbWindow *window)
 
         data->radio_ss_direction_forward = glade_xml_get_widget (data->gui, "radio_ss_direction_forward");
         data->radio_ss_direction_reverse = glade_xml_get_widget (data->gui, "radio_ss_direction_reverse");
+        data->radio_ss_direction_random = glade_xml_get_widget (data->gui, "radio_ss_direction_random");
         data->spin_ss_delay = glade_xml_get_widget (data->gui, "spin_ss_delay");
         data->toggle_ss_wrap_around = glade_xml_get_widget (data->gui, "toggle_ss_wrap_around");
-        data->toggle_ss_fullscreen = glade_xml_get_widget (data->gui, "toggle_ss_fullscreen");
 
 	btn_close  = glade_xml_get_widget (data->gui, "p_close_button");
 	btn_help   = glade_xml_get_widget (data->gui, "p_help_button");
 
 	/* Set widgets data. */
-	
+
 	/* * general */
 
 	if (eel_gconf_get_boolean (PREF_USE_STARTUP_LOCATION, FALSE))
@@ -474,15 +486,18 @@ dlg_preferences (GThumbWindow *window)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->radio_current_location), TRUE);
 	
 	if (! eel_gconf_get_boolean (PREF_USE_STARTUP_LOCATION, FALSE)) {
-		gtk_widget_set_sensitive (data->file_entry, FALSE);
+		gtk_widget_set_sensitive (data->startup_dir_filechooserbutton, FALSE);
 		gtk_widget_set_sensitive (data->btn_set_to_current, FALSE);
 	}
 
 	startup_location = eel_gconf_get_path (PREF_STARTUP_LOCATION, NULL);
-
-	if ((startup_location != NULL)
-	    && pref_util_location_is_file (startup_location)) 
-		_gtk_entry_set_filename_text (GTK_ENTRY (data->startup_dir_entry), pref_util_get_file_location (startup_location));
+	
+	if (uri_scheme_is_file (startup_location)) {
+		char *esc_uri;
+		esc_uri = gnome_vfs_escape_host_and_path_string (startup_location);
+		gtk_file_chooser_set_uri (GTK_FILE_CHOOSER (data->startup_dir_filechooserbutton), esc_uri);
+		g_free (esc_uri);
+	}
 
 	g_free (startup_location);
 
@@ -520,9 +535,6 @@ dlg_preferences (GThumbWindow *window)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->view_as_list_radiobutton), TRUE);
 
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->toggle_file_type), ! eel_gconf_get_boolean (PREF_FAST_FILE_TYPE, TRUE));
-	/* FIXME
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->toggle_show_hidden), eel_gconf_get_boolean (PREF_SHOW_HIDDEN_FILES, FALSE));
-	*/
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->toggle_show_filenames), eel_gconf_get_boolean (PREF_SHOW_FILENAMES, FALSE));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->toggle_show_comments), eel_gconf_get_boolean (PREF_SHOW_COMMENTS, TRUE));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->toggle_show_thumbs), eel_gconf_get_boolean (PREF_SHOW_THUMBNAILS, TRUE));
@@ -537,19 +549,21 @@ dlg_preferences (GThumbWindow *window)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->opt_zoom_quality_low), TRUE);
 
 	gtk_option_menu_set_history (GTK_OPTION_MENU (data->opt_zoom_change), pref_get_zoom_change ());
-	gtk_option_menu_set_history (GTK_OPTION_MENU (data->opt_transparency), image_viewer_get_transp_type (IMAGE_VIEWER (window->viewer)));
+	gtk_option_menu_set_history (GTK_OPTION_MENU (data->opt_transparency), image_viewer_get_transp_type (gth_window_get_image_viewer (GTH_WINDOW (browser))));
 
 	/* * slide show */
 
-	if (pref_get_slideshow_direction () == GTH_DIRECTION_FORWARD)
+	direction = pref_get_slideshow_direction ();
+	if (direction == GTH_DIRECTION_FORWARD)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->radio_ss_direction_forward), TRUE);
-	else
+	else if (direction == GTH_DIRECTION_REVERSE)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->radio_ss_direction_reverse), TRUE);
+	else
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->radio_ss_direction_random), TRUE);
 		
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (data->spin_ss_delay),
 				   (gfloat) eel_gconf_get_integer (PREF_SLIDESHOW_DELAY, 4));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->toggle_ss_wrap_around), eel_gconf_get_boolean (PREF_SLIDESHOW_WRAP_AROUND, FALSE));
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->toggle_ss_fullscreen), eel_gconf_get_boolean (PREF_SLIDESHOW_FULLSCREEN, TRUE));
 
 	/* Set the signals handlers. */
 
@@ -611,12 +625,6 @@ dlg_preferences (GThumbWindow *window)
 			  "toggled",
 			  G_CALLBACK (show_comments_toggled_cb),
 			  data);
-	/* FIXME
-	g_signal_connect (G_OBJECT (data->toggle_show_hidden), 
-			  "toggled",
-			  G_CALLBACK (show_hidden_toggled_cb),
-			  data);
-	*/
 
 	g_signal_connect (G_OBJECT (data->view_as_slides_radiobutton), 
 			  "toggled",
@@ -668,7 +676,8 @@ dlg_preferences (GThumbWindow *window)
 
 	/* run dialog. */
 
-	gtk_window_set_transient_for (GTK_WINDOW (data->dialog), GTK_WINDOW (window->app));
+	gtk_window_set_transient_for (GTK_WINDOW (data->dialog), 
+				      GTK_WINDOW (browser));
 	gtk_window_set_modal (GTK_WINDOW (data->dialog), FALSE);
 	gtk_widget_show_all (data->dialog);
 }
